package types

import (
	"encoding/json"
	"fmt"
	"io"
	"time"
)

// TypeObject ...
type TypeObject interface {
	EncodeToJSON(w io.Writer) error
	DecodeFromJSON(r io.Reader) error
}

// Device ...
type Device struct {
	DeviceID     string    `json:"device_id"`
	CreationDate time.Time `json:"creation_date"`
}

// EncodeToJson needs a writer to write the struct into json string
func (d *Device) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&d)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJson decode a json string from a reader into a struct
func (d *Device) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&d); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Humidity ...
type Humidity struct {
	HumidityID    string      `json:"humidity_id"`
	HumidityValue json.Number `json:"humidity_value"`
	HumidityDate  time.Time   `json:"humidity_date"`
	SensorID      string      `json:"sensor_id"`
	DeviceID      string      `json:"device_id"`
	CreationDate  time.Time   `json:"creation_date"`
}

// EncodeToJson needs a writer to write the struct into json string
func (h *Humidity) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&h)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJson decode a json string from a reader into a struct
func (h *Humidity) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&h); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Temperature ...
type Temperature struct {
	TemperatureID    string    `json:"temperature_id"`
	TemperatureValue float64   `json:"temperature_value,string"`
	TemperatureDate  time.Time `json:"temperature_date"`
	SensorID         string    `json:"sensor_id"`
	DeviceID         string    `json:"device_id"`
	CreationDate     time.Time `json:"creation_date"`
}

// EncodeToJson needs a writer to write the struct into json string
func (t *Temperature) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&t)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJson decode a json string from a reader into a struct
func (t *Temperature) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&t); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}
